/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <string.h>
#include <gnome.h>
#include <libgnome/libgnome.h>
#include <gtk/gtk.h>
#include <libgnomeui/gnome-stock.h>
#include <unistd.h>
#include <gdbm.h>
#include <errno.h>
#include "pan.h"

GDBM_FILE dbf = NULL;

void
db_close(void)
{
    if (dbf) {
        gdbm_close(dbf);
    }
}

void
db_open(void)
{
    char path[PATH_MAX];

	g_return_if_fail(Pan.active_server);

    g_snprintf(path, 256, "/%s/%s/article.gdbm", Pan.props->data_dir, Pan.active_server->name); 

    if ((dbf = gdbm_open(path, 0, GDBM_WRCREAT | GDBM_FAST, 0600, 0)) == NULL) {
        g_print(_("We couldn't open the article database:\n%s\n"), path);
		g_print(_("\nIs another copy of Pan running as this user?\n\n"));
		exit(-1);
	}
}

void
db_store_group_last(group_data * gdata, gint last)
{
    datum key, content;

    content.dptr = g_strdup_printf("%d", last);
    content.dsize = strlen(content.dptr);
    key.dptr = gdata->name;
    key.dsize = strlen(gdata->name);

    gdbm_store(dbf, key, content, GDBM_REPLACE);
    g_free(content.dptr);
}

gint
db_fetch_group_last(group_data * gdata)
{
    datum key, content;
    int last = 0;

    g_assert(gdata);

    key.dptr = gdata->name;
    key.dsize = strlen(gdata->name);

    content = gdbm_fetch(dbf, key);

    if (content.dptr)
        sscanf(content.dptr, "%d", &last);
    return(last);
}

article_data *
db_fetch_article(gchar * message_id)
{
    datum key, content;
    char **sd;
    article_data *adata = NULL;

    g_assert(message_id);

    key.dptr = message_id;
    key.dsize = strlen(message_id);

    content = gdbm_fetch(dbf, key);

    if (content.dptr == NULL)
        return NULL;

    adata = (article_data *) g_malloc(sizeof(article_data));
    memset(adata, 0, sizeof(article_data));

    sd = g_strsplit(content.dptr, "\n", 9);

    adata->message_id = g_strdup(message_id);
    adata->state = strtol(sd[0], NULL, 10);
    adata->date = strtol(sd[1], NULL, 10);
    adata->linecount = strtol(sd[2], NULL, 10);
    adata->parts = strtol(sd[3], NULL, 10);
    adata->part = strtol(sd[4], NULL, 10);
    adata->number = g_strdup(sd[5]);
    adata->subject = g_strdup(sd[6]);
    adata->author = g_strdup(sd[7]);
    adata->references = g_strdup(sd[8]);

	g_strfreev(sd);
    g_free(content.dptr);

    return(adata);
}

void
db_erase_article(gchar * message_id)
{
    datum key;

    key.dptr = message_id;
    key.dsize = strlen(message_id);

    if (gdbm_delete(dbf, key) == -1)
        g_warning("article_db_erase(%s) failed\n", message_id);
}

void 
db_store_article(article_data * adata)
{
    datum key, content;
    char *data = NULL;

    data = g_strdup_printf("%d\n%d\n%d\n%d\n%d\n%s\n%s\n%s\n%s",
                           adata->state, (int)adata->date, adata->linecount,
                           adata->parts, adata->part, adata->number, adata->subject,
                           adata->author, adata->references);
    
	content.dptr = data;
    content.dsize = strlen(data);
    key.dptr = adata->message_id;
    key.dsize = strlen(adata->message_id);

    gdbm_store(dbf, key, content, GDBM_REPLACE);

    g_free(data);
}

int
group_index_exists(group_data * gdata)
{
    char path[PATH_MAX];
	
    g_snprintf(path, 256, "/%s/%s/%s", Pan.props->data_dir, Pan.active_server->name, gdata->name); 

    if (g_file_exists(path))
        return TRUE;
	else
        return FALSE;
}

static gint
save_article_gslist(GSList * p, FILE * index, gint count, gint total)
{
    article_data *adata;
    time_t current_time;

    time(&current_time);

    for (; p != NULL; p = p->next) {
        adata = p->data;
/*		if ((current_time - adata->date) < (86400 * Pan.props->expiration)) {
*/
            db_store_article(adata);
            fprintf(index, "%d\n%s\n", (int)adata->date, adata->message_id);
            ++count;
/*        } else {
            --total;
        }
*/
        gui_set_progress((gfloat)count/(gfloat)total);
        if (adata->threads)
            count = save_article_gslist(adata->threads, index, count, total);
    }
    return count;
}

void
db_save_articles(group_data * gdata)
{
    FILE *index = NULL;
    time_t current_time;
    int n_saved = 0;
    char path[PATH_MAX];
	
    g_snprintf(path, 256, "/%s/%s/%s", Pan.props->data_dir, Pan.active_server->name, gdata->name); 

    /* Open/create the index */
    index = fopen(path, "w");
    if (index == NULL) {
        pan_error_dialog(_("Writing DB: %s\n"), strerror(errno));
        goto db_save_articles_cleanup;
    }

    fprintf(index, "%d\n", gdata->articles);
    time(&current_time);
    gui_set_progress(0);
    n_saved = save_article_gslist(gdata->article_list, index, 0, gdata->articles);
    gui_set_progress(1);

    if (n_saved > 0)
        gdata->flags |= GROUP_ONDISK;
	//gui_set_info_va(_("Saved %d article headers to disk"), n_saved);

    db_save_articles_cleanup:   
    fclose(index);
}

/***************************************************************
* db_load_articles 
****************************************************************/

GSList *
db_load_articles(group_data * gdata)
{
    GSList *l = NULL;
    time_t current_time, article_date;
    int total = 0;
    FILE *index;
    GSList *list = NULL;
	article_data *adata;
    char path[PATH_MAX];
	char *p;
	
    g_snprintf(path, 256, "/%s/%s/%s", Pan.props->data_dir, Pan.active_server->name, gdata->name); 

    /* Open the index */
    index = fopen(path, "r");
    if (index == NULL) {
        pan_error_dialog(_("Error reading article DB:\n%s\n"), strerror(errno));
        return(NULL);
    }

    gdata->articles = 0;

    time(&current_time);
    fscanf(index, "%d", &total);
    while (1) {
        fscanf(index, "%d", (int *)&article_date);
        if (feof(index))
            break;
		if ((p = fgetline(index))) {
            if ((current_time - article_date) >= (86400 * Pan.props->expiration)) {
                db_erase_article(p);
            } else if ((adata = db_fetch_article(p))) {
                adata->group = gdata;
                list = g_slist_prepend(list, adata);
                gdata->articles++;
            }
        }
        gui_set_progress((gfloat)gdata->articles/(gfloat)total);
    }
	fclose(index);

	/* Now we write the new index, minus the expired articles
     * -- if any are left*/
    if (gdata->articles) {
        index = fopen(path, "w");
        if (index == NULL) {
            pan_error_dialog(_("Error writing article DB:\n%s\n"), strerror(errno));
            return(NULL);
        }
        fprintf(index, "%d\n", gdata->articles);
        for (l = list; l; l = l->next) {
            adata = l->data;
            fprintf(index, "%d\n%s\n", (int)adata->date, adata->message_id);
        }
        gdata->flags |= GROUP_ONDISK;
    } else {
        gdata->flags &= !GROUP_ONDISK;
        unlink(path);
    }
	fclose(index);
    gui_set_progress(1);
		
	return(list);
}

void
group_index_erase(group_data * gdata)
{
	char path[PATH_MAX];
	
    g_snprintf(path, 256, "/%s/%s/%s", Pan.props->data_dir, Pan.active_server->name, gdata->name); 

    if (unlink(path) == -1)
        pan_error_dialog(_("We couldn't modify the index for '%s'\n%s"),
                         gdata->name, g_strerror(errno));
}
