/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <gnome.h>
#include <gtk/gtk.h>
#include <unistd.h>
#include <string.h>
#include <pthread.h>
#include "uulib/uudeview.h"
#include "uulib/fptools.h"
#include "pan.h"

static pthread_mutex_t online_m = PTHREAD_MUTEX_INITIALIZER;
static gint online = FALSE;
static gint must_exit = FALSE;

/**************************************************
* Cleanup Handler Stuff
***************************************************/

void (*cleanup) _ANSI_ARGS_((void *, int)) = NULL;
void *cleanup_arg  = NULL;

void
set_cleanup (void *opaque, void (*func) _ANSI_ARGS_((void *, int)))
{
  cleanup = func;
  cleanup_arg = opaque;
}

/* status = 2 when errors interrupt 
 * status = 1 when operation was interrupted
 * status = 0 when finished */

void
run_cleanup(gint status)
{
	if (cleanup && cleanup_arg) {
		(*cleanup) (cleanup_arg, status);
	}
	cleanup = NULL;
	cleanup_arg = NULL;
}

/**************************************************
* go on/off line code
***************************************************/

void
go_online(void)
{
	pthread_mutex_lock(&online_m); 
    if (online) {
		pthread_mutex_unlock(&online_m);
        return;
    } else {   
        if (connect_to_server()) {
			pthread_mutex_unlock(&online_m);
            online = FALSE;
			must_exit = FALSE;
			queue_purge();
            pthread_exit(NULL);
        } else {
            online = TRUE;
			pthread_mutex_unlock(&online_m);
        }
    }
}

void
go_offline(void)
{
    pthread_mutex_lock(&online_m);
    if (online) {
		put_server(_("Couldn't write to server"), "QUIT\r\n");
	    tcp_close();
	    online = FALSE;
		must_exit = FALSE;
	    server_menu_sensitize();
		gui_set_status_default(_("Pan - Offline"));
	    gui_clean();
		queue_purge();
	}
    pthread_mutex_unlock(&online_m);
    pthread_exit(NULL);
}

void
network_error(const gchar * format,...)
{
    va_list args;
    gchar *p = NULL;

    g_return_if_fail(format != NULL);

    va_start(args, format);
    p = g_strdup_vprintf(format, args);
    va_end(args);

    pan_lock();
    gnome_error_dialog(p);
    pan_unlock();
    g_free(p);
	/* Run any cleanup with status=2 (error) */
	run_cleanup(2);
	go_offline();
}

gint
set_group(group_data * data)
{
    gchar *buffer;

    g_assert(data);

    put_server_va(_("Error writing to socket."), "GROUP %s\r\n", data->name); 
    buffer = get_server(_("Error reading from socket."));
    if (atoi(buffer) != 211)
        return 1;
    else
        return 0;
}

/***************************************************************
* article_post
****************************************************************/

void
article_post(queue_data * qdata)
{
    gchar *buffer;
    gchar *message;

    go_online();
    message = qdata->str;

    gui_set_status(_("Posting message..."));
    put_server(_("Error writing to socket."), "POST\r\n");
    buffer = get_server(_("Error reading from socket."));
    if (atoi(buffer) == 340) {
        put_server_va(_("Error writing to socket."), "%s\r\n.\r\n", message);
		buffer = get_server(_("Error reading from socket."));
        if (atoi(buffer) == 240) {
          	gui_set_info(_("Message posted.")); 
			gui_clean();
			return;
        }
    }
	gui_clean();
    pan_error_dialog(_("Error posting message.  Server said:\n'%s'"), buffer);
}

/***************************************************************
* grouplist_download 
****************************************************************/

typedef struct _glist_c glist_c;
struct _glist_c {
	GSList *glist;
};

void
grouplist_download_cleanup(glist_c *c, gint status)
{
	switch (status) {
	case 1: /* Interrupt */
	case 2: /* Error */
		grouplist_free(c->glist);
		break;
	default: /* Success */
		break;
	}
	g_free(c);
}

void
grouplist_download(void)
{
    gchar *buffer = NULL;
    group_data *gdata = NULL;
    gint count = 0;
	glist_c *c;

	c = g_malloc(sizeof(glist_c));
	memset(c, 0 , sizeof(glist_c));
    set_cleanup(c, grouplist_download_cleanup);

	go_online();

    grouplist_free(Pan.grouplist);
	Pan.grouplist = NULL;
    gui_set_status(_("Retrieved 0 groups."));

    put_server(_("Error writing to socket."), "LIST active\r\n");
    buffer = get_server(_("Error reading from socket."));
    if (atoi(buffer) != 215)
        network_error(_("Group list could not be retrieved\nServer said: %s"), buffer);
    else {
        buffer = get_server(_("Error reading from socket."));
        gui_set_progress_activity(1);
        while (strncmp(buffer, ".\r\n", 3)) {
			if (must_exit) {
				run_cleanup(1);
				go_offline();
			}
            gdata = (group_data *) g_malloc(sizeof(group_data));
    		memset(gdata, 0, sizeof(group_data));
            gdata->name = g_strdup(strtok(buffer, " "));
            c->glist = g_slist_prepend(c->glist, gdata);
        	gui_set_progress_activity(++count);
            if ((count % 23) == 0)
                gui_set_status_va(_("Retrieved %d groups."), count);
            buffer = get_server(_("Error reading from socket."));
        }
    }
    gui_set_info_va(_("Retrieved %d groups."), count);
	gui_set_status_va(_("Sorting %d groups."), count);
    c->glist = g_slist_sort(c->glist, (GCompareFunc) compare_group_data);
	Pan.grouplist = c->glist;
    grouplist_save();
    group_clist_update();
	run_cleanup(0);
}

/***************************************************************
* articlelist_download 
****************************************************************/
	
typedef struct _alist_c alist_c;
struct _alist_c {
	GSList *alist;
};

void
articlelist_download_cleanup(alist_c *c, gint status)
{
	switch (status) {
	case 2: /* Error */
	case 1: /* Interrupt */
		articlelist_free(c->alist);
		break;
	default: /* Success */
		break;
	}
	g_free(c);
}

void
articlelist_download(queue_data * qdata)
{
	gchar *buffer = NULL;
	article_data *adata = NULL;
	gint count = 0, total, mp;
	gint first = 0;
	gint last = 0;
	gint p_last = 0;
	gchar *mid, *part, *parts;
	group_data *gdata = NULL;
	time_t current_time, date;
	gchar **sd;
	gint flags = qdata->flags;
	alist_c *c;

	c = g_malloc(sizeof(alist_c));
	memset(c, 0 , sizeof(alist_c));
    set_cleanup(c, articlelist_download_cleanup);
    gdata = qdata->gdata;
	g_free(qdata);
	
	go_online();

    if (flags & QUEUE_ARTICLELIST_DOWNLOAD_NEW) {
        if (group_index_exists(gdata)) {
            c->alist = db_load_articles(gdata);
        }
    }

    gui_set_status(_("Retrieving Article Headers"));

    put_server_va(_("Error writing to socket."), "GROUP %s\r\n", gdata->name);
    buffer = get_server(_("Error reading from socket."));

    if (strtol(buffer, NULL, 10) != 211)
        network_error(_("Couldn't select group '%s'"), gdata->name);

    sscanf(buffer, "%*d %d %d %d", &total, &first, &last);

    p_last = db_fetch_group_last(gdata);
    if (total == '0') {
		gui_set_info(_("No articles"));
        run_cleanup(0);
		return;
    }
    if ((flags & QUEUE_ARTICLELIST_DOWNLOAD_NEW) && p_last) {
        if (p_last == last) {
            gui_set_info(_("No new articles"));
        	run_cleanup(0);
			return;
        } 
		put_server_va(_("Error writing to socket."), "XOVER %d-%d\r\n", p_last, last);
    } else {
        if ((total > 150) && ((flags & 0x003F) == QUEUE_ARTICLELIST_DOWNLOAD_SAMPLE)) {
            put_server_va(_("Error writing to socket"), "XOVER %d-%d\r\n", first, first+150);
        } else {
			put_server_va(_("Error writing to socket."), "XOVER %d-%d\r\n", first, last);
		}
    }

    buffer = get_server(_("Error reading from socket."));
    if (strtol(buffer, (char **) NULL, 10) != 224)
        network_error(_("Couldn't retrieve headers\nServer said: %s"), buffer);
    time(&current_time);
    buffer = get_server(_("Error reading from socket."));
    while (strncmp(buffer, ".\r\n", 3)) {
		if (must_exit) {
			run_cleanup(1);
			go_offline();
		}
		sd = g_strsplit(buffer, "\t", 0);
		date = parse_date(sd[3]);
		if ((current_time - date) > (86400 * Pan.props->expiration)) {
			g_strfreev(sd);
        	buffer = get_server(_("Error reading from socket."));
			continue;
		}								 
		adata = (article_data *) g_malloc(sizeof(article_data));
        memset(adata, 0, sizeof(article_data));
        adata->number = g_strdup(sd[0]);
        adata->subject = g_strdup(sd[1]);
        adata->author = g_strdup(sd[2]);
        adata->date = date;
        adata->message_id = g_strdup(sd[4]);
        adata->references = g_strdup(sd[5]);
        adata->linecount = strtol(sd[7], NULL, 10);
        adata->parts = 1;
        adata->part = 1;
        adata->group = gdata;
        
		g_strfreev(sd);
		
		/* Here we look for the (n/N) or [n/N] construct in subject lines */
        mid = adata->subject;
        part = NULL;
        parts = NULL;
        mp = 0;
        while (((int) *(mid) != '\0')) {
            if (((int) *(mid) == '(') || ((int) *(mid) == '[')) {
                if (isdigit((int) *(++mid))) {
                    part = mid;
                    for (; isdigit((int) *(mid)); mid++);
                    if ((int) *(mid) == '/') {
                        if (isdigit((int) *(++mid))) {
                            parts = mid;
                            if (strtol(parts, (char **) NULL, 10) >= strtol(part, (char **) NULL, 10)) {
                                for (; isdigit((int) *(mid)); mid++);
                                if (((int) *(mid) == ')') || ((int) *(mid) == ']')) {
                                    mp = 1;
                                    break;
                                }
                            }
                        }
                    }
                }
            }
            mid++;
        }

        // Handle Multiparts
        if (mp) {
            adata->parts = strtol(parts, (char **) NULL, 10);
            adata->part = strtol(part, (char **) NULL, 10);
        }
        // Add the article to the article list
        c->alist = g_slist_prepend(c->alist, adata);

        gui_set_progress((gfloat)++count/(gfloat)total);
        buffer = get_server(_("Error reading from socket."));
 	}
	/* Check to see if all articles were expired */
	if (c->alist == NULL) {
        gui_set_info(_("All articles were expired"));
		run_cleanup(0);
		return;
	}

    gui_set_status(_("Saving Article List"));
    
    db_store_group_last(gdata, last);
	gdata->articles += count;

	gui_set_info_va(_("Retrieved %d article headers"), count);
	gui_set_status(_("Threading Article List"));
    gui_set_progress(0);

    c->alist = thread_list_by_multipart(c->alist, gdata);
	if (must_exit) {
		run_cleanup(1);
		go_offline();
	}
    c->alist = thread_list_by_subject(c->alist, gdata);
	if (must_exit) {
		run_cleanup(1);
		go_offline();
	}
    c->alist = thread_list_by_reference(c->alist, gdata);
	if (must_exit) {
		run_cleanup(1);
		go_offline();
	}

	gdata->article_list = c->alist;
    db_save_articles(gdata);
 	group_clist_update_row(gdata);
	
	if (flags & QUEUE_FOREGROUND) {
        gui_set_status(_("Displaying article list"));
        article_ctree_update((group_data *) gdata);
    }
	run_cleanup(0);
}

/***************************************************************
* article_download 
***************************************************************/

typedef struct _ad_c ad_c;
struct _ad_c {
	queue_data *qdata;
	article_data *adata;
};

void
article_download_cleanup(ad_c *c, gint status)
{
	if (c->adata) {
		c->adata->state &= ~STATE_DOWNLOAD_QUEUED;	
		if (status == 2)
			c->adata->state |= STATE_ERROR;
		else 
			c->adata->state &= ~STATE_ERROR;
		article_update(c->adata);
    	article_free(c->adata);
	}
	g_free(c->qdata);
	g_free(c);
}

void
article_download(queue_data * qdata)
{
    gchar *buffer;
    group_data *gdata;
   	gint count;
	ad_c *c;

	c = g_malloc(sizeof(ad_c));
	memset(c, 0 , sizeof(ad_c));
    set_cleanup(c, article_download_cleanup);
    gdata = qdata->gdata;
	c->qdata = qdata;

    go_online();
    
    if ((c->adata = db_fetch_article(qdata->str)) == NULL) {
        gui_set_info_va(_("Couldn't find article header for message_id: %s"), qdata->str);
        run_cleanup(2);
		return;
    }

    gui_set_status_va(_("Downloading article '%s'"), c->adata->subject);

    if (set_group(gdata))
        network_error(_("Couldn't select group '%s'"), gdata->name);

    put_server_va(_("Error writing to socket."), "HEAD %s\r\n", c->adata->number);
    buffer = get_server(_("Error reading from socket."));

    if (strtol(buffer, (char **) NULL, 10) != 221) {
		run_cleanup(0);
		return;
	}

	c->adata->header = g_strdup("");
	buffer = get_server(_("Error reading from socket."));
	while (strncmp(buffer, ".\r\n", 3)) {
		if (must_exit) {
			run_cleanup(2);
			go_offline();
		}
		c->adata->header = g_realloc(c->adata->header, strlen(c->adata->header) + strlen(buffer) + 1);
		c->adata->header = strcat(c->adata->header, buffer);
		buffer = get_server(_("Error reading from socket."));
	}
	
	put_server_va(_("Error writing to socket."), "BODY %s\r\n", c->adata->number);
    buffer = get_server(_("Error reading from socket."));
    if (strtol(buffer, (char **) NULL, 10) != 222) {
		run_cleanup(2);
		return;
	}
	
	c->adata->body = g_strdup("");
	count = 0;
	buffer = get_server(_("Error reading from socket."));
	while (strncmp(buffer, ".\r\n", 3)) {
		if (must_exit) {
			run_cleanup(1);
			go_offline();
		}
		c->adata->body = g_realloc(c->adata->body, strlen(c->adata->body) + strlen(buffer) + 1);
		c->adata->body = strcat(c->adata->body, buffer);
    	gui_set_progress((gfloat)count++/(gfloat)c->adata->linecount);
		buffer = get_server(_("Error reading from socket."));
	}
    gui_set_progress(1);

    if (qdata->flags & QUEUE_FOREGROUND)
        update_text((article_data *) c->adata);

	c->adata->state |= STATE_READ;
	run_cleanup(0);
}

/***************************************************************
* UUdeview Lib Stuff 
***************************************************************/

void
UUD_message_callback(void *opaque, char *msg, int level)
{
    if ((level == UUMSG_WARNING) || (level == UUMSG_ERROR)) {
        pan_error_dialog(_("An error occured during decoding.\n%s"), msg);
    }
}

int
UUD_busy_callback(void *opaque, uuprogress * progress)
{
	gfloat t = (gfloat)1 / (gfloat)3;
		
	switch (progress->action) {
    case UUACT_SCANNING:
        gui_set_progress((gfloat)progress->percent / (gfloat)100 * t);
        return(0);
    case UUACT_DECODING:
        gui_set_progress(((gfloat)progress->percent / (gfloat)100 * t) + t);
        return(0);
    case UUACT_COPYING:
        gui_set_progress(((gfloat)progress->percent / (gfloat)100 * t) + t + t);
        return(0);
    default:
        g_warning("Reached default in UUD_busy_callback\n");
        return(0);
    }
}

char *
UUD_file_name_filter(void *opaque, char *fname)
{
	char *ptr;

	if (fname == NULL)
		return NULL;

	/*
	* strip directory information
	*/

	if ((ptr = _FP_strrchr (fname, '/')) != NULL)
		return ptr+1;
	else if ((ptr = _FP_strrchr (fname, '\\')) != NULL)
		return ptr+1;

	return fname;
}

void
decode(article_data *adata, gchar *filename)
{
	uulist *item;

	chdir(Pan.props->download_dir);
    UUInitialize();
    UUSetMsgCallback(adata, UUD_message_callback);
    UUSetBusyCallback(adata, UUD_busy_callback, 1);
    UUSetFNameFilter(adata, UUD_file_name_filter);
    if ((UULoadFile(filename, NULL, 0)) != UURET_OK) {
        pan_error_dialog(_("Decoded couldn't load temporary file:\n%s"), filename);
    } else if ((item = UUGetFileListItem(0)) != NULL) {
        if ((item->state & UUFILE_OK) != 0) {
            if ((UUDecodeFile(item, NULL)) != UURET_OK) {
                pan_error_dialog(_("Error decoding '%s'"), adata->subject);
                gui_set_info_va(_("Error decoding '%s'"), adata->subject);
            } else {
                adata->state |= STATE_DECODED;
                gui_set_info_va(_("Successfully decoded '%s'"), UUFNameFilter(item->filename));
            }
        } else {
			pan_error_dialog(_("Error decoding '%s'"), adata->subject);
			gui_set_info_va(_("Error decoding '%s'"), adata->subject);
		}
    } 
	UUCleanUp();
}

/***************************************************************
* article_decode 
***************************************************************/

typedef struct _adec_c adec_c;
struct _adec_c {
	queue_data *qdata;
	article_data *adata;
	gchar *filename;
	FILE *tmp;
};

void
article_decode_cleanup(adec_c *c, gint status)
{
	if (c->adata) {
		c->adata->state &= ~STATE_DECODE_QUEUED;
		if (status == 2) /* Error */
			c->adata->state |= STATE_ERROR;
		else
			c->adata->state &= ~STATE_ERROR;
		article_update(c->adata);
    	article_free(c->adata);
	}
	if (c->filename) {
		if (c->tmp) {
			fclose(c->tmp);
		}
		unlink(c->filename);
		g_free(c->filename);
	}
	g_free(c->qdata);
	g_free(c);
}

void
article_decode(queue_data * qdata)
{
    gchar *buffer = NULL;
    group_data *gdata = NULL;
    gint count;
	adec_c *c;
	
	c = g_malloc(sizeof(adec_c));
	memset(c, 0 , sizeof(adec_c));
    set_cleanup(c, article_decode_cleanup);
    gdata = qdata->gdata;
	c->qdata = qdata;
	
	go_online();

	if ((c->adata = db_fetch_article(qdata->str)) == NULL) {
		gui_set_info_va(_("Couldn't find article header for message_id: %s"), qdata->str);
		run_cleanup(2);
		return;
    }
    
    c->filename = tempnam(Pan.props->temp_dir, "pan");
    c->tmp = fopen(c->filename, "w");
	
    if (c->tmp == NULL)
        network_error(_("Couldn't open temporary file %s"), c->filename);
    
    if (set_group(gdata))
        network_error(_("Couldn't select group '%s'"), gdata->name);
       	
	gui_set_status_va(_("Downloading article body '%s'"), c->adata->subject);
    gui_set_progress(0);
    
	put_server_va(_("Error writing to socket."), "BODY %s\r\n", c->adata->number);
    buffer = get_server(_("Error reading from socket."));
    if (strtol(buffer, NULL, 10) != 222) {
		run_cleanup(2);
		return;
	}

	count = 1;
	buffer = get_server(_("Error reading from socket."));
	while (strncmp(buffer, ".\r\n", 3)) {
		if (must_exit) {
			run_cleanup(1);
			go_offline();
		}
		fputs(buffer, c->tmp);
		buffer = get_server(_("Error reading from socket."));
		gui_set_progress(percent(count++, c->adata->linecount));
	}
	gui_set_progress(1);
	fclose(c->tmp);
	c->tmp = NULL;
	
	gui_set_status_va(_("Decoding article: %s"), c->adata->subject);
    gui_set_progress(0);

    decode(c->adata, c->filename);

	run_cleanup(0);
}

/***************************************************************
* multipart_decode 
***************************************************************/

typedef struct _mdec_c mdec_c;
struct _mdec_c {
	queue_data *qdata;
	article_data *fadata;
	article_data *adata;
	gchar *filename;
	FILE *tmp;
	gchar **sd;
};

void
multipart_decode_cleanup(mdec_c *c, gint status)
{
	if (c->fadata) {
		c->fadata->state &= ~STATE_DECODE_QUEUED;
		if (status == 2) /* Error */
			c->fadata->state |= STATE_ERROR;
		else
			c->fadata->state &= ~STATE_ERROR;
		article_update(c->fadata);
    	article_free(c->fadata);
	}
	if (c->adata)
		article_free(c->adata);
	if (c->filename) {
		if (c->tmp) {
			fclose(c->tmp);
		}
		unlink(c->filename);
		g_free(c->filename);
	}
	g_strfreev(c->sd);
	g_free(c->qdata);
	g_free(c);
}

void
multipart_decode(queue_data * qdata)
{
    gchar *buffer;
    group_data *gdata;
    gint i = 0;
    gint count;
	mdec_c *c;

	c = g_malloc(sizeof(mdec_c));
	memset(c, 0 , sizeof(mdec_c));
    set_cleanup(c, multipart_decode_cleanup);
    gdata = qdata->gdata;
	c->qdata = qdata;

    go_online();
    
    c->sd = g_strsplit(qdata->str, "\n", -1);
    g_assert(c->sd);

    /* Find the (fake) article header for the
     * multipart node */
    if ((c->fadata = db_fetch_article(c->sd[2])) == NULL) {
		gui_set_info_va(_("Couldn't find article header for message_id: %s"), qdata->str);
        run_cleanup(2);
		return;
    }

    c->filename = tempnam(Pan.props->temp_dir, "pan");

    if ((c->tmp = fopen(c->filename, "w")) == NULL) {
        gui_set_info(_("Multipart decode failed: Couldn't open tempfile"));
        run_cleanup(2);
		return;
    }
    if (set_group(gdata))
        network_error(_("Couldn't select group '%s'"), gdata->name);
    for (i = 2; c->sd[i]; i++) {
		if ((c->adata = db_fetch_article(c->sd[i])) == NULL) {
			gui_set_info_va(_("Couldn't find article header for message_id: %s"), c->sd[i]);
        	run_cleanup(2);
			return;
        }
		gui_set_status_va(_("Downloading part %d of %d: %s"), c->adata->part, c->adata->parts, c->adata->subject);
        gui_set_progress(0);

		put_server_va(_("Error writing to socket."), "body %s\r\n", c->adata->number);
        buffer = get_server(_("Error reading from socket."));

        if (strtol(buffer, (char **) NULL, 10) != 222) {
            run_cleanup(2);
			return;
		}

        count = 1;
        buffer = get_server(_("Error reading from socket."));

        while (strncmp(buffer, ".\r\n", 3)) {
			if (must_exit) {
				run_cleanup(1);
				return;
			}
            fputs(buffer, c->tmp);
            buffer = get_server(_("Error reading from socket."));
            gui_set_progress(percent(count++,c->adata->linecount));
        }
        article_free(c->adata);
		c->adata = NULL;
    }
	fclose(c->tmp);
	c->tmp = NULL;

    gui_set_progress(1);
    gui_set_status_va(_("Decoding multipart '%s'"), c->sd[1]);
    gui_set_progress(0);

    decode(c->fadata, c->filename);

	run_cleanup(0);
}

/* I might implement something like this 
 * someday
 *
void
send_keepalive()
{
    gchar *buffer;

    if (online_check())
        return;

    if (put_server("help\r\n") == -1)
        network_error(_("Error writing to socket."));
    if ((buffer = get_server()) == NULL)
        network_error(_("Error reading from socket."));
    if (strtol(buffer, (char **) NULL, 10) == 100) {
        while (strncmp(buffer, ".\r\n", 3)) {
            if ((buffer = get_server()) == NULL)
                network_error(_("Error reading from socket."));
        }
        return;
    } else {
        network_error(_("Problem with HELP no-op."));
    }
    return;
}
*/

int
connect_to_server(void)
{
    gchar *buffer;
    gint response;

    if (Pan.active_server == NULL) {
        pan_error_dialog(_("There is no active server.\nPlease configure one."));
        return(1);
    }
    gui_set_status(_("Connecting to server"));

    if (tcp_open(Pan.active_server, "nntp") < 0) {
        gui_clean();
        pan_error_dialog(_("Could not connect to server."));
        return(1);
    }
    if ((buffer = get_server(NULL)) == NULL) {
        gui_clean();
        pan_error_dialog(_("Could not connect to server."));
        return(1);
    }
    gui_set_status_default(_("Pan - Online"));

    response = strtol(buffer, (char **) NULL, 10);
    if ((response != 201) && (response != 200)) {
        gui_clean();
        pan_error_dialog(_("Could not connect to server."));
        return(1);
    }
    if (response == 201)
        Pan.active_server->posting = 0;
    else
        Pan.active_server->posting = 1;
    return(0);
}

void
stop_all_online_tasks(void)
{
	pthread_mutex_lock(&online_m);
    if (online)
        must_exit = TRUE;
	pthread_mutex_unlock(&online_m);
}
