/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <stdio.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <pwd.h>
#include <unistd.h>
#include <gnome.h>
#include <gtk/gtk.h>
#include "pan.h"

typedef struct _PostWindow PostWindow;
struct _PostWindow {
    GtkWidget *window;
    GtkWidget *from;
    GtkWidget *organization;
    GtkWidget *newsgroups;
    GtkWidget *subject;
    GtkWidget *text;
    article_data *article;
};

void
edit_cut_cb(GtkWidget * widget, PostWindow * pw)
{
    gtk_editable_cut_clipboard(GTK_EDITABLE(pw->text));
}

void
edit_copy_cb(GtkWidget * widget, PostWindow * pw)
{
    gtk_editable_copy_clipboard(GTK_EDITABLE(pw->text));
}

void
edit_paste_cb(GtkWidget * widget, PostWindow * pw)
{
    gtk_editable_paste_clipboard(GTK_EDITABLE(pw->text));
}

void
destroy_post_window(GtkWidget * widget, PostWindow * pw)
{
    gtk_widget_destroy(pw->newsgroups);
    gtk_widget_destroy(pw->subject);
    gtk_widget_destroy(pw->text);
    g_free(pw);
}

static void 
close_window(GtkWidget * widget, PostWindow * pw)
{
    gtk_widget_destroy(pw->window);
}

static gint
check_from_syntax(gchar * s)
{
    return 1;
}

static gint
check_newsgroups_syntax(gchar * s)
{
    return 1;
}

static gchar *
generate_references(PostWindow * pw)
{
#define GNKSA_LENGTH 998
    gchar buf[GNKSA_LENGTH + 1];
    gchar *p;
    gchar *refs;
    gchar *new_refs;
    unsigned int total_len, len;

    p = extract_header(pw->article, "References");
    if (p)
        refs = g_strconcat(p, " ", extract_header(pw->article, "Message-ID"), NULL);
    else
        refs = g_strconcat(extract_header(pw->article, "Message-ID"), NULL);
    g_free(p);

    len = strlen(refs);
    if (len <= GNKSA_LENGTH) {
        return refs;
    }
    /* The reference header is too long so start truncating */
    /* We have to preserve the first message-id */
    p = strchr(refs, '>');
    if (p == NULL)
        return NULL;
    p++;
    len = p - refs;
    if (len >= GNKSA_LENGTH) {
        return refs;
    }
    strncpy(buf, refs, len);
    buf[len] = 0;
    new_refs = g_strconcat(buf, " ", NULL);
    total_len = len + 1;

    refs += len;
    if ((*refs == '\t') || (*refs == ' '))
        refs++;         /* already handled space */

    /* Now use message-ids at end of the line */
    p = refs + strlen(refs);

    while (1) {
        if (*p == '<') {
            if (strlen(p) + total_len > GNKSA_LENGTH) {
                p = strchr(p + 1, '<');
                if (p == NULL)
                    p = refs;
                break;
            }
        }
        if (p == refs)
            break;
        p--;
    }

    new_refs = g_strconcat(new_refs, p, NULL);
    g_free(refs);
    return new_refs;
}

gchar *
generate_message_id()
{
    unsigned long now;
    unsigned long randy;
    static unsigned char baseid[64];
    gchar *b, *t, tmp[32];
    char *chars32 = "0123456789abcdefghijklmnopqrstuv";
    static unsigned long last_now;

    while (1) {
        time((time_t *) & now);
        if (now != last_now)
            break;
        sleep(1);
    }
    last_now = now;
    srand(now + getpid());
    randy = (unsigned long) rand();

    b = baseid;
    t = tmp;
    while (now) {
        *t++ = chars32[now & 0x1F];
        now = now >> 5;
    }

    while (t > tmp) {
        t--;
        *b++ = *t;
    }
    *b++ = '.';

    t = tmp;
    while (randy) {
        *t++ = chars32[randy & 0x1F];
        randy = randy >> 5;
    }
    while (t > tmp) {
        t--;
        *b++ = *t;
    }

    *b++ = '@';
    return(char *) baseid;
}

gchar *
get_host_name(void)
{
    gchar *ptr;
    gchar hostname[MAXHOSTNAMELEN + 1];

    hostname[0] = '\0';

    if ((gethostname(hostname, sizeof(hostname))) == -1) {
        if ((ptr = getenv("HOST")) != NULL)
            strncpy(hostname, ptr, MAXHOSTNAMELEN);
        else {
            if ((ptr = getenv("HOSTNAME")) != NULL)
                strncpy(hostname, ptr, MAXHOSTNAMELEN);
            else
                hostname[0] = '\0';
        }
    }
    hostname[MAXHOSTNAMELEN] = '\0';
    ptr = strtok(hostname, ".");
    return(g_strdup(ptr));
}

gchar *
get_fqdn(gchar * host)
{
    char name[MAXHOSTNAMELEN + 2];
    char line[1025];
    char *cp, *domain;
    static char fqdn[1024];
    struct hostent *hp;
    struct in_addr in;
    FILE *inf;

    *fqdn = '\0';
    domain = NULL;

    name[MAXHOSTNAMELEN] = '\0';
    if (host) {
        if (strchr(host, '.'))
            return host;
        strncpy(name, host, MAXHOSTNAMELEN);
    } else {
        if (gethostname(name, MAXHOSTNAMELEN))
            return(NULL);
    }

    if ('0' <= *name && *name <= '9') {
        in.s_addr = inet_addr(name);
        if ((hp = gethostbyaddr((char *) &in.s_addr, 4, AF_INET)))
            in.s_addr = (*hp->h_addr);
        return(hp && strchr(hp->h_name, '.') ? hp->h_name : (gchar *) inet_ntoa(in));
    }
    if ((hp = gethostbyname(name)) && !strchr(hp->h_name, '.'))
        if ((hp = gethostbyaddr(hp->h_addr, hp->h_length, hp->h_addrtype)))
            in.s_addr = (*hp->h_addr);

    sprintf(fqdn, "%s", hp
            ? strchr(hp->h_name, '.')
            ? hp->h_name
            : (gchar *) inet_ntoa(in)
            : "");
    if (!*fqdn || (fqdn[strlen(fqdn) - 1] <= '9')) {
        *fqdn = '\0';
        inf = fopen("/etc/resolv.conf", "r");
        if (inf) {
            while (fgets(line, 1024, inf)) {
                line[1024] = '\0';
                (void) Pan_strtrim(line);
                if (strncmp(line, "domain ", 7) == 0) {
                    domain = line + 7;
                    break;
                }
                if (strncmp(line, "search ", 7) == 0) {
                    domain = line + 7;
                    cp = strchr(domain, ' ');
                    if (cp)
                        *cp = '\0';
                    break;
                }
            }
            if (domain)
                sprintf(fqdn, "%s.%s", name, domain);
            fclose(inf);
        }
    }
    return(fqdn);
}

void 
send_post(GtkWidget * widget, PostWindow * pw)
{
    gchar *subject, *from, *newsgroups, *message, *path, *organization;
    static char *host = NULL;
    static char *uname = NULL;
    static char *fqdn = NULL;

    if (pw == NULL) {
        g_error("WTF!! pw == NULL\n");
    }
    if (host == NULL)
        host = get_host_name();
    if (fqdn == NULL)
        fqdn = get_fqdn(host);
    if (uname == NULL)
        uname = g_get_user_name();

    path = g_strconcat(host, "!", uname, NULL);

    from = gtk_entry_get_text(GTK_ENTRY(pw->from));
    newsgroups = gtk_entry_get_text(GTK_ENTRY(pw->newsgroups));
    subject = gtk_entry_get_text(GTK_ENTRY(pw->subject));
    organization = gtk_entry_get_text(GTK_ENTRY(pw->organization));

    if (!check_from_syntax(from))
        g_error("Bad From. Do something!");
    if (!check_newsgroups_syntax(newsgroups))
        g_error("Bad Newsgroups. Do something!");

    message = g_strconcat("From: ", from, "\n",
                          "Path: ", path, "\n",
                          "Newsgroups: ", newsgroups, "\n",
                          "Subject: ", subject, "\n",
                          "Message-ID: <", generate_message_id(), fqdn, ">\n",
                          NULL);
    if (pw->article) {
        message = g_strconcat(message, "References: ", generate_references(pw), "\n", NULL);
    }
    message = g_strconcat(message, "Date: ", generate_date(), "\n"
                          "Organization: ", organization, "\n",
                          "X-Newsreader: ", "Pan ", VERSION, "\n\n",
                          gtk_editable_get_chars(GTK_EDITABLE(pw->text), 0, -1),
                          NULL);

    queue_add_post (message);
    close_window(NULL, pw);
}

static GtkWidget *
create_postWindow_toolbar(PostWindow * pw)
{
    GtkWidget *toolbar;
    GtkWidget *toolbarbutton;

    toolbar = gtk_toolbar_new(0, 0);
    gtk_widget_realize(pw->window);

    toolbarbutton = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
                                            _("Post"), _("Post this Message"), NULL,
                                            gnome_stock_pixmap_widget(pw->window, GNOME_STOCK_PIXMAP_MAIL_SND),
                                            GTK_SIGNAL_FUNC(send_post), pw);
    GTK_WIDGET_UNSET_FLAGS(toolbarbutton, GTK_CAN_FOCUS);

    gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));

    toolbarbutton = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
                                            _("Spelling"), _("Check Spelling"), NULL,
                                            gnome_stock_pixmap_widget(pw->window, GNOME_STOCK_PIXMAP_SPELLCHECK),
                                            NULL, pw);
    GTK_WIDGET_UNSET_FLAGS(toolbarbutton, GTK_CAN_FOCUS);

    gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));

    toolbarbutton = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
                                            _("Print"), _("Print"), NULL,
                                            gnome_stock_pixmap_widget(pw->window, GNOME_STOCK_PIXMAP_PRINT),
                                            NULL, pw);
    GTK_WIDGET_UNSET_FLAGS(toolbarbutton, GTK_CAN_FOCUS);

    gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));

    toolbarbutton = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
                                            _("Cancel"), _("Cancel"), NULL,
                                            gnome_stock_pixmap_widget(pw->window, GNOME_STOCK_PIXMAP_CLOSE),
                                            GTK_SIGNAL_FUNC(close_window), pw);
    GTK_WIDGET_UNSET_FLAGS(toolbarbutton, GTK_CAN_FOCUS);

    //gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), balsa_app.toolbar_style);
    gtk_widget_show(toolbar);
    return toolbar;
}

static GtkWidget *
create_postwindow_menu(PostWindow * pw)
{
    GtkWidget *menubar, *w, *menu;
    GtkAccelGroup *accel;
    guchar key;
    guint8 mod;

    accel = gtk_accel_group_new();
    menubar = gtk_menu_bar_new();
    gtk_widget_show(menubar);

    menu = gtk_menu_new();

    w = gnome_stock_menu_item(GNOME_STOCK_MENU_OPEN, _("Open..."));
    gtk_widget_show(w);
    if (gnome_stock_menu_accel(GNOME_STOCK_MENU_OPEN, &key, &mod))
        gtk_widget_add_accelerator(w, "activate", accel, key, mod, 0);

    gtk_menu_append(GTK_MENU(menu), w);

    w = gnome_stock_menu_item(GNOME_STOCK_MENU_SAVE, _("Save"));
    gtk_widget_show(w);
    if (gnome_stock_menu_accel(GNOME_STOCK_MENU_SAVE, &key, &mod))
        gtk_widget_add_accelerator(w, "activate", accel, key, mod, 0);

    gtk_menu_append(GTK_MENU(menu), w);
    w = gnome_stock_menu_item(GNOME_STOCK_MENU_SAVE_AS, _("Save as..."));
    gtk_widget_show(w);
    if (gnome_stock_menu_accel(GNOME_STOCK_MENU_SAVE_AS, &key, &mod))
        gtk_widget_add_accelerator(w, "activate", accel, key, mod, 0);

    gtk_menu_append(GTK_MENU(menu), w);

    w = gtk_menu_item_new();
    gtk_widget_show(w);
    gtk_menu_append(GTK_MENU(menu), w);

    w = gnome_stock_menu_item(GNOME_STOCK_MENU_PRINT, _("Print..."));
    gtk_widget_show(w);
    if (gnome_stock_menu_accel(GNOME_STOCK_MENU_PRINT, &key, &mod))
        gtk_widget_add_accelerator(w, "activate", accel, key, mod, 0);

    gtk_menu_append(GTK_MENU(menu), w);

    w = gnome_stock_menu_item(GNOME_STOCK_MENU_BLANK, _("Setup Page..."));
    gtk_widget_show(w);
    gtk_menu_append(GTK_MENU(menu), w);

    w = gtk_menu_item_new();
    gtk_widget_show(w);
    gtk_menu_append(GTK_MENU(menu), w);

    w = gnome_stock_menu_item(GNOME_STOCK_MENU_CLOSE, _("Cancel"));
    gtk_widget_show(w);
    gtk_widget_add_accelerator(w, "activate", accel, 'A', GDK_CONTROL_MASK,
                               GTK_ACCEL_VISIBLE);
    gtk_menu_append(GTK_MENU(menu), w);
    gtk_signal_connect(GTK_OBJECT(w), "activate",
                       GTK_SIGNAL_FUNC(close_window), pw);

    w = gtk_menu_item_new_with_label(_("File"));
    gtk_widget_show(w);
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(w), menu);
    gtk_menu_bar_append(GTK_MENU_BAR(menubar), w);

    menu = gtk_menu_new();

    w = gnome_stock_menu_item(GNOME_STOCK_MENU_CUT, _("Cut"));
    gtk_widget_show(w);
    gtk_widget_add_accelerator(w, "activate", accel, 'X', GDK_CONTROL_MASK,
                               GTK_ACCEL_VISIBLE);
    gtk_signal_connect(GTK_OBJECT(w), "activate",
                       GTK_SIGNAL_FUNC(edit_cut_cb), pw);
    gtk_menu_append(GTK_MENU(menu), w);

    w = gnome_stock_menu_item(GNOME_STOCK_MENU_COPY, _("Copy"));
    gtk_widget_show(w);
    gtk_widget_add_accelerator(w, "activate", accel, 'C', GDK_CONTROL_MASK,
                               GTK_ACCEL_VISIBLE);
    gtk_signal_connect(GTK_OBJECT(w), "activate",
                       GTK_SIGNAL_FUNC(edit_copy_cb), pw);
    gtk_menu_append(GTK_MENU(menu), w);

    w = gnome_stock_menu_item(GNOME_STOCK_MENU_PASTE, _("Paste"));
    gtk_widget_show(w);
    gtk_widget_add_accelerator(w, "activate", accel, 'V', GDK_CONTROL_MASK,
                               GTK_ACCEL_VISIBLE);
    gtk_signal_connect(GTK_OBJECT(w), "activate",
                       GTK_SIGNAL_FUNC(edit_paste_cb), pw);
    gtk_menu_append(GTK_MENU(menu), w);

    w = gtk_menu_item_new_with_label(_("Edit"));
    gtk_widget_show(w);
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(w), menu);
    gtk_menu_item_right_justify(GTK_MENU_ITEM(w));
    gtk_menu_bar_append(GTK_MENU_BAR(menubar), w);

    menu = gtk_menu_new();

    w = gnome_stock_menu_item(GNOME_STOCK_MENU_MAIL_SND, _("Post Message"));
    gtk_widget_show(w);
    gtk_widget_add_accelerator(w, "activate", accel, 'P', 0, GTK_ACCEL_VISIBLE);
    gtk_menu_append(GTK_MENU(menu), w);
    gtk_signal_connect(GTK_OBJECT(w), "activate", GTK_SIGNAL_FUNC(send_post), pw);

    w = gtk_menu_item_new_with_label(_("Message"));
    gtk_widget_show(w);
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(w), menu);
    gtk_menu_bar_append(GTK_MENU_BAR(menubar), w);

    menu = gtk_menu_new();

    w = gnome_stock_menu_item(GNOME_STOCK_MENU_ABOUT, _("Contents"));
    gtk_widget_show(w);
    gtk_menu_append(GTK_MENU(menu), w);

    w = gtk_menu_item_new_with_label(_("Help"));
    gtk_widget_show(w);
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(w), menu);
    gtk_menu_item_right_justify(GTK_MENU_ITEM(w));
    gtk_menu_bar_append(GTK_MENU_BAR(menubar), w);

    gtk_window_add_accel_group(GTK_WINDOW(pw->window), accel);
    return menubar;
}

static GtkWidget *
create_info_pane(PostWindow * pw)
{
    GtkWidget *table;
    GtkWidget *label;
    gchar *temp;

    table = gtk_table_new(4, 2, FALSE);
    gtk_table_set_row_spacings(GTK_TABLE(table), 2);
    gtk_table_set_col_spacings(GTK_TABLE(table), 2);

    label = gtk_label_new("From:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
                     GTK_FILL, GTK_FILL, 4, 0);

    pw->from = gtk_entry_new();
    gtk_table_attach(GTK_TABLE(table), pw->from, 1, 2, 0, 1,
                     GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

    if (Pan.props->email) {
        if (Pan.props->full_name) {
            temp = g_strconcat(Pan.props->email, " (",
                               Pan.props->full_name, ")", NULL);
        } else {
            temp = g_strdup(Pan.props->email);
        }
        gtk_entry_set_text(GTK_ENTRY(pw->from), temp);
        g_free(temp);
    }
    label = gtk_label_new("Organization:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
                     GTK_FILL, GTK_FILL, 4, 0);

    pw->organization = gtk_entry_new();
    gtk_table_attach(GTK_TABLE(table), pw->organization, 1, 2, 1, 2,
                     GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);

    if (Pan.props->organization)
        gtk_entry_set_text(GTK_ENTRY(pw->organization), Pan.props->organization);

    label = gtk_label_new("Newsgroups:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
                     GTK_FILL, GTK_FILL, 4, 0);

    pw->newsgroups = gtk_entry_new();
    gtk_table_attach(GTK_TABLE(table), pw->newsgroups, 1, 2, 2, 3,
                     GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);
    if (pw->article) {
        gtk_entry_set_text(GTK_ENTRY(pw->newsgroups), extract_header(pw->article, "Newsgroups"));
    } else if (Pan.displayed_group) {
        gtk_entry_set_text(GTK_ENTRY(pw->newsgroups), Pan.displayed_group->name);
    }
    label = gtk_label_new("Subject:");
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4,
                     GTK_FILL, GTK_FILL, 4, 0);

    pw->subject = gtk_entry_new();
    gtk_table_attach(GTK_TABLE(table), pw->subject, 1, 2, 3, 4,
                     GTK_FILL | GTK_EXPAND, GTK_FILL, 4, 0);
    if (pw->article) {
        if (((pw->article->subject[0] == 'R') || (pw->article->subject[0] == 'r')) &&
            ((pw->article->subject[1] == 'E') || (pw->article->subject[1] == 'e')) &&
            (pw->article->subject[2] == ':') && (pw->article->subject[3] == ' ')) {
            gtk_entry_set_text(GTK_ENTRY(pw->subject), pw->article->subject);
        } else {
            gtk_entry_set_text(GTK_ENTRY(pw->subject), g_strconcat("Re: ", pw->article->subject, NULL));
        }
    }
    return table;
}

static GtkWidget *
create_text_area(PostWindow * pw)
{
    GtkWidget *table;
    GtkWidget *hscrollbar;
    GtkWidget *vscrollbar;
    GdkFont *font;
    GtkStyle *style;

    style = gtk_style_new();
    font = gdk_font_load("-adobe-courier-medium-r-*-*-*-120-*-*-*-*-iso8859-1");
    style->font = font;

    table = gtk_table_new(2, 2, FALSE);

    pw->text = gtk_text_new(NULL, NULL);
    gtk_text_set_editable(GTK_TEXT(pw->text), TRUE);
    gtk_text_set_word_wrap(GTK_TEXT(pw->text), TRUE);
    gtk_widget_set_style(pw->text, style);
    gtk_widget_set_usize(pw->text, (72 * 7) + (2 * pw->text->style->klass->xthickness) + 8, -1);
    gtk_widget_show(pw->text);
    gtk_table_attach_defaults(GTK_TABLE(table), pw->text, 0, 1, 0, 1);
    hscrollbar = gtk_hscrollbar_new(GTK_TEXT(pw->text)->hadj);
    GTK_WIDGET_UNSET_FLAGS(hscrollbar, GTK_CAN_FOCUS);
    gtk_table_attach(GTK_TABLE(table), hscrollbar, 0, 1, 1, 2,
                     GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);

    vscrollbar = gtk_vscrollbar_new(GTK_TEXT(pw->text)->vadj);
    GTK_WIDGET_UNSET_FLAGS(vscrollbar, GTK_CAN_FOCUS);
    gtk_table_attach(GTK_TABLE(table), vscrollbar, 1, 2, 0, 1,
                     GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);

    return table;
}

void 
post_window(GtkWidget * widget, gint reply)
{
    PostWindow *pw;
    GtkWidget *vbox;
    gchar *body, *tmp, *tmp2, c;

    pw = g_malloc(sizeof(PostWindow));

    if (reply) {
        // Followup
        pw->article = displayed_article();
        pw->window = gnome_app_new("pan", "Followup");
    } else {
        // New Post
        pw->article = NULL;
        pw->window = gnome_app_new("pan", "Post");
    }

    gtk_signal_connect(GTK_OBJECT(pw->window), "destroy", (GtkSignalFunc) destroy_post_window, pw);
    gtk_signal_connect(GTK_OBJECT(pw->window), "delete_event", (GtkSignalFunc) gtk_false, NULL);

    vbox = gtk_vbox_new(FALSE, 0);
    gnome_app_set_contents(GNOME_APP(pw->window), vbox);
    gnome_app_set_menus(GNOME_APP(pw->window),
                        GTK_MENU_BAR(create_postwindow_menu(pw)));
    gnome_app_set_toolbar(GNOME_APP(pw->window),
                          GTK_TOOLBAR(create_postWindow_toolbar(pw)));
    gtk_box_pack_start(GTK_BOX(vbox), create_info_pane(pw), FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox), gtk_hseparator_new(), FALSE, FALSE, 2);
    gtk_box_pack_start(GTK_BOX(vbox), create_text_area(pw), TRUE, TRUE, 0);
    if (reply) {
        gtk_widget_realize(pw->text);
        gtk_text_freeze(GTK_TEXT(pw->text));
        tmp = g_strdup_printf(_("Nearly %d seconds after the Epoch, %s wrote:\n"),
                              (int) pw->article->date, pw->article->author);
        gtk_text_insert(GTK_TEXT(pw->text), NULL, &pw->text->style->black, NULL, tmp, -1);
        g_free(tmp);
        body = g_strdup(pw->article->body);
        tmp = body;
        tmp2 = strchr(tmp, '\n');
        while (tmp2) {
            c = *(tmp2 + 1);
            *(tmp2 + 1) = '\0';
            gtk_text_insert(GTK_TEXT(pw->text), NULL, &pw->text->style->black, NULL, "> ", 2);
            gtk_text_insert(GTK_TEXT(pw->text), NULL, &pw->text->style->black, NULL, tmp, -1);
            *(tmp2 + 1) = c;
            tmp = tmp2 + 1;
            tmp2 = strchr(tmp, '\n');
        }
        g_free(body);
        gtk_text_thaw(GTK_TEXT(pw->text));
    }
    gtk_widget_show_all(pw->window);
}

void 
followup_window()
{
    if (displayed_article() == NULL) {
        return;
    }
    post_window(NULL, 1);
}
