/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <gnome.h>
#include <gtk/gtk.h>
#include <unistd.h>
#include <string.h>
#include <pthread.h>
#include "uulib/uudeview.h"
#include "uulib/fptools.h"
#include "pan.h"

GSList *queue = NULL;
GSList *queue_last = NULL;
pthread_mutex_t qlock = PTHREAD_MUTEX_INITIALIZER;
gint q_active = FALSE;

void
queue_append(queue_data *qdata)
{
    GSList *l;
	
	l = g_slist_alloc();
	l->data = qdata;
	
	if (queue) {
		g_assert(queue_last);
		g_slist_concat(queue_last, l);
		queue_last = l;
	} else {
        queue = queue_last = l;
	}
}

void
queue_prepend(queue_data *qdata)
{
	if (queue_last)
		queue = g_slist_prepend(queue, qdata);
	else
		queue_last = queue = g_slist_prepend(queue, qdata); 
}

queue_data *
queue_get_next(void)
{
    queue_data *qdata;

    pthread_mutex_lock(&qlock);
    if (queue) {
        qdata = (queue_data *) queue->data;
        queue = g_slist_remove_link(queue, queue);
		if (queue == NULL)
			queue_last = NULL;
        pthread_mutex_unlock(&qlock);
        return qdata;
    } else {
        pthread_mutex_unlock(&qlock);
        return NULL;
    }
}

void
queue_purge(void)
{
    GSList *p;
    queue_data *qdata;
    article_data *adata;
    gint in_mem = 0;
    gchar **sd;

    pthread_mutex_lock(&qlock);

    for (p = queue; p; p = p->next) {
        qdata = p->data;
        switch (qdata->flags & 0x003F) {
        case QUEUE_ARTICLE_DOWNLOAD:
        case QUEUE_ARTICLE_DECODE:
            if ((adata = article_in_memory(qdata->str)))
                in_mem = 1;
            else
                adata = db_fetch_article(qdata->str);
            if (adata) {
				adata->state &= ~STATE_DOWNLOAD_QUEUED;
				adata->state &= ~STATE_DECODE_QUEUED;
				article_update(adata);
				if (!in_mem)
					g_free(adata);
			}
            break;
        case QUEUE_MULTIPART_DECODE:
            sd = g_strsplit(qdata->str, "\n", 1);
            g_assert(sd[1]);
            if ((adata = article_in_memory(sd[1])))
                in_mem = 1;
            else
                adata = db_fetch_article(sd[1]);
            if (adata) {
				adata->state &= ~STATE_DECODE_QUEUED;
            	article_update(adata);
            	if (!in_mem)
                	g_free(adata);
			}
            break;
        }
    }
    g_slist_free(queue);
    queue = NULL;
    queue_last = NULL;
	q_active = FALSE;
    pthread_mutex_unlock(&qlock);
}

void
stop_queue_processing(void)
{
    pthread_mutex_lock(&qlock);
    g_slist_free(queue);
    queue = NULL;
    queue_last = NULL;
    q_active = FALSE;
    server_menu_sensitize();
	pthread_mutex_unlock(&qlock);
	go_offline();
}

void
queue_processing(void)
{
    queue_data *qdata;

	while ((qdata = queue_get_next())) {
		switch (qdata->flags & 0x003F) {
        case QUEUE_ARTICLE_DOWNLOAD:
            article_download(qdata);
            break;
        case QUEUE_ARTICLE_DECODE:
            article_decode(qdata);
            break;
        case QUEUE_ARTICLELIST_LOAD:
            articlelist_load(qdata);
            break;
        case QUEUE_ARTICLELIST_DOWNLOAD_ALL:
        case QUEUE_ARTICLELIST_DOWNLOAD_NEW:
        case QUEUE_ARTICLELIST_DOWNLOAD_SAMPLE:
            articlelist_download(qdata);
            break;
        case QUEUE_GROUPLIST_DOWNLOAD:
            grouplist_download();
            break;
        case QUEUE_ARTICLE_POST:
            article_post(qdata);
            break;
        case QUEUE_MULTIPART_DECODE:
            multipart_decode(qdata);
            break;
        default:
            g_error("Reached default in queue_processing()\n");
            break;
        }
		gui_clean();
    }
	stop_queue_processing();
}

void
start_queue_processing(void)
{
    pthread_mutex_lock(&qlock);
    if (q_active == FALSE) {
		server_menu_desensitize();
    	pthread_create(&Pan.thread, NULL, (void *) &queue_processing, NULL);
    	pthread_detach(Pan.thread);
		q_active = TRUE;
    	pthread_mutex_unlock(&qlock);
	} else {
    	pthread_mutex_unlock(&qlock);
	}
}

void
queue_add_post(gchar *message)
{
    queue_data *qdata;

    qdata = (queue_data *) g_malloc(sizeof(queue_data));
	memset(qdata, 0, sizeof(queue_data));
    qdata->str = message;
    qdata->flags = QUEUE_ARTICLE_POST;
    pthread_mutex_lock(&qlock);
    queue_prepend(qdata);
    pthread_mutex_unlock(&qlock);
    start_queue_processing();
}

void
queue_add_grouplist_download(void)
{
    queue_data *qdata;

    qdata = (queue_data *) g_malloc(sizeof(queue_data));
	memset(qdata, 0, sizeof(queue_data));
    qdata->flags = QUEUE_GROUPLIST_DOWNLOAD;
    pthread_mutex_lock(&qlock);
    queue = g_slist_prepend(queue, qdata);
    pthread_mutex_unlock(&qlock);
    start_queue_processing();
}

void
queue_add_article_download(article_data * adata, guint flags)
{
    queue_data *qdata;
    GSList *temp_list;

    qdata = (queue_data *) g_malloc(sizeof(queue_data));
	memset(qdata, 0, sizeof(queue_data));
    qdata->gdata = adata->group;
    qdata->str = g_strdup(adata->message_id);
    qdata->flags = flags;
	temp_list = g_slist_alloc();
	temp_list->data = qdata;
    pthread_mutex_lock(&qlock);
    if (flags & QUEUE_FOREGROUND) {
        queue_prepend(qdata);
    } else if (flags & QUEUE_BACKGROUND) {
        queue_append(qdata);
    } else {
        g_error("Unreachable condition reached\n");
    }
    pthread_mutex_unlock(&qlock);
    start_queue_processing();
}

void
queue_add_multipart_download(article_data * adata, guint flags)
{
    queue_data *qdata;
    article_data *adata0;
    GSList *l;
	gchar *tmp;

    g_assert(adata && adata->threads && flags);

    qdata = (queue_data *) g_malloc(sizeof(queue_data));
	memset(qdata, 0, sizeof(queue_data));
    qdata->gdata = adata->group;

    qdata->str = g_strdup_printf("%d\n%s\n%s", adata->linecount, adata->subject,
                                 adata->message_id);
    for (l = adata->threads; l; l = l->next) {
        adata0 = l->data;
        if (adata0->part > 0) {
            tmp = qdata->str;
            qdata->str = g_strconcat(tmp, "\n", adata0->message_id, NULL);
            g_free(tmp);
        }
    }
    qdata->flags = flags;
    pthread_mutex_lock(&qlock);
    queue_append(qdata);
    pthread_mutex_unlock(&qlock);
    start_queue_processing();
}

void
queue_add_articlelist_download(group_data * gdata, guint flags)
{
    queue_data *qdata;

    qdata = (queue_data *) g_malloc(sizeof(queue_data));
	memset(qdata, 0, sizeof(queue_data));
    qdata->gdata = gdata;
    qdata->flags = flags;
    pthread_mutex_lock(&qlock);
    if (flags & QUEUE_FOREGROUND) {
        queue_prepend(qdata);
    } else if (flags & QUEUE_BACKGROUND) {
        queue_append(qdata);
    } else {
        g_error("Unreachable condition reached\n");
    }
    pthread_mutex_unlock(&qlock);
    start_queue_processing();
}
